import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { DistributeTransactionDto } from './dto/distribute-transaction.dto';
import { OrderTransactionDto } from './dto/order-transaction.dto';
import { InjectRepository } from '@nestjs/typeorm';
import { Transactions } from './entities/transactions.entity';
import { Connection, EntityManager, Repository } from 'typeorm';
import { COA } from './entities/coa.entity';
import { TransactionType } from './entities/transaction-type.entity';
import { TransactionJournal } from './entities/transaction-journal.entity';
import { CoaService } from './coa.service';
import * as uuid from "uuid";
import { uniq } from "lodash";
import {Decimal} from 'decimal.js';
import {
  balanceType,
  coaType,
  statusTransaction,
  typeTransaction,
} from '../helper/enum-list';
import { ProductService } from '../product/product.service';
import * as irsService from '../helper/irs-service';
import { CreateJournalDto } from './dto/create-journal.dto';

interface JournalEntry {
  coa_id: string;
  debit?: string;
  credit?: string;
}


@Injectable()
export class TransactionService {
  constructor(
    @InjectRepository(Transactions)
    private transactionRepository: Repository<Transactions>,
    @InjectRepository(TransactionType)
    private transactionTypeRepository: Repository<TransactionType>,
    @InjectRepository(TransactionJournal)
    private transactionJournalRepository: Repository<TransactionJournal>,
    @InjectRepository(COA)
    private coaRepository: Repository<COA>,
    private coaService: CoaService,
    private productService: ProductService,
    private connection: Connection,
  ) {}

  async distributeDeposit(distributeTransactionDto: DistributeTransactionDto) {
    // GET COA
    const coaSender = await this.coaService.findByUser(
      'id_user',
      coaType.WALLET,
    );
    const coaReciever = await this.coaService.findByUser(
      'id_user',
      coaType.WALLET,
    );

    await this.connection.transaction(async (manager) => {
      //INSERT TRANSACTION
      const transactionSaved = await manager.insert(Transactions, {
        amount: distributeTransactionDto.amount,
        user: 'id_user',
        userDestination: distributeTransactionDto.destination,
        status: statusTransaction.SUCCESS,
        type: typeTransaction.DISTRIBUTION,
      });

      //INSERT TRANSACTION JOURNAL FOR SENDER
      const journalSender = await manager.insert(TransactionJournal, {
        amount: distributeTransactionDto.amount,
        transaction: transactionSaved.identifiers[0],
        coa: coaSender,
        type: balanceType.CREDIT,
      });

      //INSERT TRANSACTION JOURNAL FOR RECEIVER
      const journalReceiver = await manager.insert(TransactionJournal, {
        amount: distributeTransactionDto.amount,
        transaction: transactionSaved.identifiers[0],
        coa: coaReciever,
        type: balanceType.DEBIT,
      });

      //UPDATE AMOUNT COA SENDER
      coaSender.amount = coaSender.amount - distributeTransactionDto.amount;
      await manager.save(coaSender);

      coaReciever.amount = coaReciever.amount + distributeTransactionDto.amount;
      await manager.save(coaReciever);
    });

    return true;
  }

  async orderTransaction(orderTransactionDto: OrderTransactionDto) {
    const coaAccount = await this.coaService.findByUser(
      'id_user',
      coaType.WALLET,
    );

    //GET PRODUCT
    const product = await this.productService.findOne(
      orderTransactionDto.productCode,
    );

    if (coaAccount.amount <= product.price) {
      throw new HttpException(
        {
          statusCode: HttpStatus.INTERNAL_SERVER_ERROR,
          error: `Transaction Failed because saldo not enough`,
        },
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }

    try {
      const orderIRS = await irsService.createTransaction(
        orderTransactionDto.productCode,
        orderTransactionDto.destination,
      );

      if (orderIRS.success) {
        await this.connection.transaction(async (manager) => {
          //INSERT TRANSACTION
          const transactionSaved = await manager.insert(Transactions, {
            amount: product.price,
            user: 'id_user',
            status: statusTransaction.SUCCESS,
            type: typeTransaction.ORDER,
          });

          //INSERT TRANSACTION JOURNAL
          const journalSender = await manager.insert(TransactionJournal, {
            amount: product.price,
            transaction: transactionSaved.identifiers[0],
            coa: coaAccount,
            type: balanceType.CREDIT,
          });

          //UPDATE AMOUNT COA
          coaAccount.amount = coaAccount.amount - product.price;
          await manager.save(coaAccount);
        });
      } else {
        throw new HttpException(
          {
            statusCode: HttpStatus.INTERNAL_SERVER_ERROR,
            error: `Transaction Failed because ${orderIRS.msg}`,
          },
          HttpStatus.INTERNAL_SERVER_ERROR,
        );
      }
    } catch (e) {
      throw e;
    }

    return true;
  }

  async accountingTransaction(createJournalDto: CreateJournalDto ) {

    createJournalDto.transactionId = createJournalDto.transactionId ?? uuid.v4();

    let creditSum = createJournalDto.journals.map(it => it.credit).filter(it => it).reduce((a, b) => a.plus(b), new Decimal(0));
    let debitSum = createJournalDto.journals.map(it => it.debit).filter(it => it).reduce((a, b) => a.plus(b), new Decimal(0));
    let coaIds = uniq(createJournalDto.journals.map(it => it.coa_id));
    
    if (!creditSum.equals(debitSum)) {
        throw new Error(`credit and debit doesn't match`);
    }

    const coas = await this.coaRepository.findByIds(coaIds);

    let transaction: Transactions;

    if(createJournalDto.createTransaction) {
        transaction = new Transactions();
        transaction.id = createJournalDto.transactionId;
        transaction.type = createJournalDto.type;
        transaction.amount = createJournalDto.amount;
        transaction.user = createJournalDto.userId;
        transaction.status = createJournalDto.transactionStatus;

        await this.transactionRepository.save(transaction);
    } else {
        transaction = await this.transactionRepository.findOneOrFail(createJournalDto.transactionId);
    }

    await Promise.all(createJournalDto.journals.map(journal => {
        const coa = coas.find(it => it.id === journal.coa_id);

        if (!coa) {
            throw new Error(`coa ${journal.coa_id} not found`);
        }

        const journalEntry = new TransactionJournal();
        journalEntry.coa = coa;
        journalEntry.type = journal.debit ? balanceType.DEBIT : balanceType.CREDIT;
        journalEntry.amount = (journal.debit) ? journal.debit : journal.credit;
        journalEntry.transaction = transaction;

        return this.transactionJournalRepository.save(journalEntry);
    }));

    await Promise.all(coaIds.map(coaId => {
        let journalPerCoa = createJournalDto.journals.filter(journal => journal.coa_id == coaId);

        let creditSum = journalPerCoa.map(it => it.credit).filter(it => it).reduce((a, b) => a.plus(b), new Decimal(0));
        let debitSum = journalPerCoa.map(it => it.debit).filter(it => it).reduce((a, b) => a.plus(b), new Decimal(0));

        let coa = coas.find(it => it.id.toLowerCase() === coaId.toLowerCase());

        let balance = new Decimal(coa.amount);

        if (coa.balanceType === balanceType.DEBIT) {
            balance = balance.plus(debitSum.minus(creditSum));
        } else if (coa.balanceType === balanceType.CREDIT) {
            balance = balance.plus(creditSum.minus(debitSum));
        }

        const diff = balance.minus(new Decimal(coa.amount));

        return createJournalDto.transactionalEntityManager
            .createQueryBuilder()
            .update(COA)
            .set({
                amount: () => "amount + " + diff.toString()
            })
            .where("id = :id", { id: coa.id })
            .execute();
    }));

    return transaction;
  }
}
